package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.AlarmRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AlarmRule;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/equipmentBinding")
public class EquipmentBindingController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    CommFeedback fb = new CommFeedback();

    Gson gson = new Gson();

    @Autowired
    private ConfigService configService;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private AlarmRedisService alarmRedisService;

    @Autowired
    private AlarmService alarmService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private I18nController  i18n;


    @ApiOperation(value = "绑定gateWay和tdl",notes = "绑定gateWay和tdl" +
            "   SN：gateWay编号，" +
            "   type：gateWay类型，" +
            "   mode：1--运输/2--仓库，" +
            "   TDLSN：tdl编号，" +
            "   transportationNo：运输编号，" +
            "   warehouseNo：仓库编号," +
            "   alarmType:预警类型" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/binding")
    public Object binding(@RequestBody BindingVo bindingVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(bindingVo.getgSN())){
            fb.setCode(0);
            fb.setMessage("网关编号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(bindingVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage("Tdl编号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(bindingVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage("预警类型不能为空");
            return gson.toJson(fb);
        }
        if(bindingVo.getMode() == null){
            fb.setCode(0);
            fb.setMessage("使用场景不能为空");
            return gson.toJson(fb);
        }
        if(bindingVo.getMode() == 1){
            if(StringUtils.isEmpty(bindingVo.getTransportationNo())){
                fb.setCode(0);
                fb.setMessage("运输编号不能为空");
                return gson.toJson(fb);
            }
            bindingVo.setWarehouseNo("");
        }else if(bindingVo.getMode() == 2){
            if(StringUtils.isEmpty(bindingVo.getWarehouseNo())){
                fb.setCode(0);
                fb.setMessage("仓库编号不能为空");
                return gson.toJson(fb);
            }
            bindingVo.setTransportationNo("");
        }
        if(StringUtils.isEmpty(bindingVo.getType())){
            bindingVo.setType(gatewayService.getBySN(bindingVo.getgSN(),user.getCompanyNo()));
        }
        //获取gateWay的信息
        ResultGatewayVo resultGatewayVo =  gatewayService.getBySNAndType(bindingVo.getgSN(),bindingVo.getType());
        if(resultGatewayVo == null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        if(resultGatewayVo.getUseScene() !=0 && resultGatewayVo.getUseScene() !=bindingVo.getMode()){
            fb.setCode(0);
            fb.setMessage("该网关所属环境与当前不符");
            return gson.toJson(fb);
        }
        //判断当前tdl是否有网关在使用
        ResultTDLDeviceVo resultTDLDeviceVo = tdlDeviceService.getByTDLSN(bindingVo.getTDLSN());
        if(!StringUtils.isEmpty(resultTDLDeviceVo.getGatewaySN())){
            fb.setCode(0);
            fb.setMessage("该TDL已经被使用");
            return gson.toJson(fb);
        }
//        if(StringUtils.isEmpty(bindingVo.getTransportationNo()) && !StringUtils.isEmpty(bindingVo.getWarehouseNo())){
//            bindingVo.setTransportationNo("");
//        }else if(StringUtils.isEmpty(bindingVo.getWarehouseNo()) && !StringUtils.isEmpty(bindingVo.getTransportationNo())){
//            bindingVo.setWarehouseNo("");
//        }
        Integer gpsPeriod = null;
        Integer gprsPeriod = null;
        if(bindingVo.getMode() == 2){
            gpsPeriod = 43200;
            gprsPeriod= 60;
        }else{
            gpsPeriod=5;
            gprsPeriod=15;
        }
        //开始绑定
        //获取gateway下的所有tdl的编号
        List<Float> tempL = new ArrayList<>();
        List<Float> tempH = new ArrayList<>();
        List<Float> humiL = new ArrayList<>();
        List<Float> humiH = new ArrayList<>();
        List<Integer> pressL = new ArrayList<>();
        List<Integer> pressH = new ArrayList<>();
        List<Integer> sugEnergy = new ArrayList<>();
        List<Float> accThreshold = new ArrayList<>();
        List<Integer> timeThreshold = new ArrayList<>();
        List<Integer> tiltThreshold = new ArrayList<>();
        ResultAlarmVo alarmVo =  alarmService.getByType(bindingVo.getAlarmType(),user.getCompanyNo());
        tempL.add(alarmVo.getTemMin());
        tempH.add(alarmVo.getTemMax());
        humiL.add(alarmVo.getHumidityMin());
        humiH.add(alarmVo.getHumidityMax());
        pressL.add(255);
        pressH.add(255);
        sugEnergy.add(1);
        accThreshold.add(Float.valueOf(alarmVo.getShock()));
        timeThreshold.add(5);
        tiltThreshold.add((int)Double.parseDouble(alarmVo.getTilt().toString()));
        List<String> devList = tdlDeviceService.getByGatewaySN(bindingVo.getgSN(),bindingVo.getType());
        devList.add("TDL-"+bindingVo.getTDLSN());
        ConfigCMDVo configCMDVo = new ConfigCMDVo("config",gpsPeriod,gprsPeriod,devList,tempL,tempH,humiL,humiH,pressL,pressH,sugEnergy,accThreshold,timeThreshold,tiltThreshold,bindingVo.getMode(),System.currentTimeMillis()/1000l);
        Map<Object,Object> map=new HashMap<>();
        map.put("companyNo",user.getCompanyNo());
        map.put("transportationNo",bindingVo.getTransportationNo());
        map.put("warehouseNo",bindingVo.getWarehouseNo());
        map.put("useScene",bindingVo.getMode());
        map.put("gSN",bindingVo.getgSN());
        map.put("gType",bindingVo.getType());
        map.put("alarmType",bindingVo.getAlarmType());
        map.put("message",gson.toJson(configCMDVo));
        map.put("flag",false);
        map.put("resp",null);
        map.put("state",1);
        map.put("untie",false);
        map.put("TDLSN",bindingVo.getTDLSN());
        map.put("createTime",System.currentTimeMillis());
        configService.addConfig(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("配置网关成功")){
            fb.setCode(1);
            fb.setMessage("配置网关成功");
            //根据报警类型货报警信息
            AlarmRule.saveAlarmRule("TDL/"+bindingVo.getType()+"/" +bindingVo.getgSN()+"/Data",bindingVo.getTDLSN(),bindingVo.getAlarmType(),user.getCompanyNo(),alarmService,alarmRedisService);
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }



    @ApiOperation(value = "重置tdl",notes = "重置tdl")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/untie")
    public Object untie(@RequestBody String TDLSN){
        if(StringUtils.isEmpty(TDLSN)){
            fb.setCode(0);
            fb.setMessage("TDL序列号不能为空");
            return  gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(TDLSN);
        TDLSN=(String)jsonObject.get("TDLSN");
        //判断当前tdl是否有网关在使用
        ResultTDLDeviceVo resultTDLDeviceVo = tdlDeviceService.getByTDLSN(TDLSN);
        List<String> devList = tdlDeviceService.getByGatewaySN(resultTDLDeviceVo.getGatewaySN(),resultTDLDeviceVo.getGatewayType());
        ConfigVo configVo =configService.getConfig(resultTDLDeviceVo.getGatewaySN(),resultTDLDeviceVo.getGatewayType());
        ConfigCMDVo config = gson.fromJson(configVo.getMessage(),ConfigCMDVo.class);
        ConfigCMDVo configCMDVo ;
        Boolean boo = false;
        if(devList.size()>1){
            devList.remove("TDL-"+TDLSN);
            configCMDVo = new ConfigCMDVo("config",config.getGpsPeriod(),config.getGprsPeriod(),devList,config.getMode(),System.currentTimeMillis()/1000l);
        }else{
            configCMDVo = null;
            boo = true;
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("gSN",resultTDLDeviceVo.getGatewaySN());
        map.put("gType",resultTDLDeviceVo.getGatewayType());
        map.put("message",configCMDVo == null ? "": gson.toJson(configCMDVo));
        map.put("untie",boo);
        map.put("TDLSN",TDLSN);
        map.put("endTime",System.currentTimeMillis());
        configService.updateConfig(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("解除网关成功")){
            fb.setCode(1);
            fb.setMessage("解除tdl成功");
            String key ="TDL/"+resultTDLDeviceVo.getGatewayType()+"/" +resultTDLDeviceVo.getGatewaySN() +"/Data";
            alarmRedisService.delKey(key);
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return  gson.toJson(fb);
    }


    @ApiOperation(value = "解绑网关",notes = "解绑网关")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/untieGateway")
    public  Object untieGateway(@RequestBody AddWarehouseGatewayVo addWarehouseGatewayVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(addWarehouseGatewayVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage("仓库编号不能为空");
            return  gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(addWarehouseGatewayVo.getWarehouseNo(),user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage("仓库编号不存在");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseGatewayVo.getgSN())){
            fb.setCode(0);
            fb.setMessage("网关编号不能为空");
            return gson.toJson(fb);
        }
        if( gatewayService.getBySN(addWarehouseGatewayVo.getgSN(),user.getCompanyNo())==null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        //判断该网关是否属于当前仓库
        Integer i =  warehouseService.getByWarehouseNoAndSN(addWarehouseGatewayVo.getWarehouseNo(),addWarehouseGatewayVo.getgSN());
        if(i == 0){
            fb.setCode(0);
            fb.setMessage("该网关不属于该仓库");
            return gson.toJson(fb);
        }
        //获取网关编号
        String  type = gatewayService.getBySN(addWarehouseGatewayVo.getgSN(),user.getCompanyNo());
        Map<Object,Object> map=new HashMap<>();
        map.put("warehouseNo",addWarehouseGatewayVo.getWarehouseNo());
        map.put("gSN",addWarehouseGatewayVo.getgSN());
        map.put("gType",type);
        map.put("endTime",System.currentTimeMillis());
        warehouseService.untieGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if(msg.get("msg").equals("删除网关成功")){
            fb.setCode(1);
            fb.setMessage(msg.get("msg").toString());
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return  gson.toJson(fb);
    }


}
